@@ -10,8 +10,11 @@ The built-in regression models submodule for the gaussian_process module.
 
 
 import numpy as np
+from ..utils import deprecated
 
 
+@deprecated("The function constant of regression_models is "
+            "deprecated in version 0.19.1 and will be removed in 0.22.")
 def constant(x):
     """
     Zero order polynomial (constant, p = 1) regression model.
@@ -36,6 +39,8 @@ def constant(x):
     return f
 
 
+@deprecated("The function linear of regression_models is "
+            "deprecated in version 0.19.1 and will be removed in 0.22.")
 def linear(x):
     """
     First order polynomial (linear, p = n+1) regression model.
@@ -60,6 +65,8 @@ def linear(x):
     return f
 
 
+@deprecated("The function quadratic of regression_models is "
+            "deprecated in version 0.19.1 and will be removed in 0.22.")
 def quadratic(x):
     """
     Second order polynomial (quadratic, p = n*(n-1)/2+n+1) regression model.
